@@ -1,7 +1,9 @@
 #!/usr/bin/perl -w
 
-# Copyright (C) 2002  Jrg Platte <joergplatte@gmx.de>
-#                 and Cajus Pollmeier <pollmeier@gonicus.de>
+# Copyright (C) 2002-2014 Jörg Platte <joergplatte@gmx.de>
+# Copyright (C) 2002-2014 Cajus Pollmeier <pollmeier@gonicus.de>
+# Copyright (C) 2013-2014 Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
+# Copyright (C) 2013-2014 Alex Owen <r.alex.owen@gmail.com>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -29,6 +31,8 @@ use NetAddr::IP;
 my $SignalCatched=0;
 
 my $configfile="/etc/uif/uif.conf";
+my $configfile6="/etc/uif/uif6.conf";
+my $ipv6=0;
 
 my @mapping = (	[ 'n', 'uifid', 'Name' ],
 		[ 's', 'uifsource', 'Source'],
@@ -117,6 +121,24 @@ sub readConfig {
 				} else {
 					die "invalid line in section include: $line\n";
 				}
+			} elsif ($state eq 'INCLUDE6') {
+				if ($ipv6) {
+					if ($line =~ /^\s*\"(.+)\"$/) {
+						my $file = $1;
+						readConfig ($file, $Networks, $Services, $Interfaces, $Protocols, $Rules, $Id, $Sysconfig);
+					} else {
+						die "invalid line in section include6: $line\n";
+					}
+				}
+			} elsif ($state eq 'INCLUDE4') {
+				if ($ipv6) {} else {
+					if ($line =~ /^\s*\"(.+)\"$/) {
+						my $file = $1;
+						readConfig ($file, $Networks, $Services, $Interfaces, $Protocols, $Rules, $Id, $Sysconfig);
+					} else {
+						die "invalid line in section include4: $line\n";
+					}
+				}
 			} elsif ($state eq 'NETWORK') {
 				if ($line =~ /^\s*([a-zA-Z0-9_-]+)\s+(.*)$/) {
 					$$Networks{$1}.="$2 ";
@@ -266,13 +288,21 @@ sub simplifyNetworks {
 		foreach (@networks) {
 			my $ip;
 			if ($_ =~ /(^[^=]+)=([^=]+)$/ ) {
-				$ip=new NetAddr::IP ($1) || die "not a valid network: $1\n";
+				if ($ipv6) {
+					$ip=NetAddr::IP->new6($1) || die "not a valid network: $1\n";
+				} else {
+					$ip=NetAddr::IP->new($1) || die "not a valid network: $1\n";
+				}
 				if (!exists($macs{$ip})) {
 					$macs{$ip}=[];
 				}
 				push (@{$macs{$ip}}, $2);
 			} else {
-				$ip=new NetAddr::IP ($_) || die "not a valid network: $_\n";
+				if ($ipv6) {
+					$ip=NetAddr::IP->new6($_) || die "not a valid network: $1\n";
+				} else {
+					$ip=NetAddr::IP->new($_) || die "not a valid network: $1\n";
+				}
 				$onlymacs=0;
 			}
 			push(@netobjects, $ip);
@@ -391,12 +421,12 @@ sub toRange {
 		}
 	} else {
 		die "invalid $proto service: $range:\n$rule\n";
-	}	
+	}
 }
 
 sub validateData {
 	my ($Networks, $Services, $Interfaces, $Protocols, $Rules, $Sysconfig, $Marker) = @_;
-	
+
 	validateSysconfig $Sysconfig;
 
 	my $key;
@@ -419,7 +449,7 @@ sub validateData {
 		$$Marker{$key} = resolveHashentries($$Marker{$key}, $Marker);
 	}
 
-## marken auf plausibilitt prfen
+## marken auf plausibilität prüfen
 
 	my $rule;
 	foreach $rule (@$Rules) {
@@ -514,6 +544,20 @@ sub validateData {
 						$$rule{"${hashentry}-not"} = 1;
 					}
 					$position=$2;
+
+					# support IPv4-only/IPv6-only rules
+					if ($position =~ /^.*\((.+)\)$/) {
+						my $only_proto = $1;
+						$position =~ s/\((.+)\)$//;
+						if (($ipv6) && ($only_proto eq "4")) {
+							print "IPv6 setup: Skipping IPv4-only rule for network \"$position\"\n";
+							next;
+						} elsif ((! $ipv6) && ($only_proto eq "6")) {
+							print "IPv4 setup: Skipping IPv6-only rule for network \"$position\"\n";
+							next;
+						}
+					}
+
 					if (exists($$Networks{$position})) {
 						my $network;
 						foreach $network (split (/\s+/, $$Networks{$position})) {
@@ -647,7 +691,7 @@ sub validateData {
 						my @destinationrange;
 						my @sourcedestination;
 						my %other;
-						my $service; 
+						my $service;
 						foreach $service (split (/\s+/, $protocols{"$serviceprefix$proto"})) {
 							$service eq '' && next;
 							if ($service  =~ /^([^\/]*)\/([^\/]*)$/) {
@@ -812,7 +856,7 @@ sub validateData {
 					} else {
 						$$rule{'Reject'}=1;
 					}
-				} elsif ($flag =~ /^account(\((.+)\)|)$/) { 
+				} elsif ($flag =~ /^account(\((.+)\)|)$/) {
 					if ($$rule{'Table'} eq 'nat') {
 						die "can't use accounting with nat:\n$$rule{'Rule'}\n";
 					}
@@ -829,7 +873,7 @@ sub validateData {
 					} else {
 						$$rule{'Reject'}='default';
 					}
-				} elsif ($flag =~ /^limit(\((.*)\)|)$/) { 
+				} elsif ($flag =~ /^limit(\((.*)\)|)$/) {
 					if ($$rule{'Action'} eq 'DROP') {
 						die "limiting packets in reject/drop rule makes no sense:\n$$rule{'Rule'}\n";
 					}
@@ -842,7 +886,7 @@ sub validateData {
 							if (checkLimit $1) {
 								$$rule{'Limit'}=$1;
 								if ($2) {
-									# no need to check burst since it 
+									# no need to check burst since it
 									# is guaranteed to be either empty
 									# or digits only (plus leading colon).
 									# Empty results in other part of if
@@ -932,7 +976,7 @@ sub genRuleDump {
 				}
 			} else {
 				$action="-j MYREJECT";
-				
+
 			}
 			$logaction="REJECT";
 		} elsif ($$rule{'Action'} eq "TCPMSS") {
@@ -1000,9 +1044,17 @@ sub genRuleDump {
 			my $type;
 			foreach $type (@{$$rule{'ICMP'}}) {
 				if ($type eq 'all') {
-					push (@protocol, "$not -p icmp");
+					if ($ipv6) {
+						push (@protocol, "$not -p icmpv6");
+					} else {
+						push (@protocol, "$not -p icmp");
+					}
 				} else {
-					push (@protocol, "-p icmp -m icmp $not --icmp-type $type");
+					if ($ipv6) {
+						push (@protocol, "-p icmpv6 -m icmpv6 $not --icmpv6-type $type");
+					} else {
+						push (@protocol, "-p icmp -m icmp $not --icmp-type $type");
+					}
 				}
 			}
 		}
@@ -1147,7 +1199,7 @@ sub genRuleDump {
 				$logid=$name;
 			}
 			push (@$table, "-A $chain -m limit --limit $$Sysconfig{'LogLimit'} --limit-burst $$Sysconfig{'LogBurst'} -j LOG --log-prefix \"$$Sysconfig{'LogPrefix'} $logaction ($logid): \" --log-level $$Sysconfig{'LogLevel'} --log-tcp-options --log-ip-options");
-			push (@$table, "-A $chain $action"); 
+			push (@$table, "-A $chain $action");
 			$action="-j $chain";
 		}
 		if (exists($$rule{'Accounting'})) {
@@ -1222,12 +1274,13 @@ sub genRuleDump {
 				$type="-A ${id}_$level";
 				$$chains{"${id}_$level"}=1;
 				$level++;
-			}			
+			}
 		}
 	}
 
 	my $entry;
 	foreach $entry (qw(mangle filter nat)) {
+		if ($entry eq "nat" && $ipv6 == 1) {next};
 		my $chain;
 		push (@$Listing, "*$entry");
 		if ($entry eq 'filter') {
@@ -1245,13 +1298,21 @@ sub genRuleDump {
 				push (@$Listing, "-A STATE$_ -m state --state INVALID -j STATELESS$_");
 				push (@$Listing, "-A STATE$_ -j ACCOUNTING$_");
 				push (@$Listing, "-A STATE$_ -m state --state ESTABLISHED,RELATED -j ACCEPT");
-				push (@$Listing, "-A STATE$_ -m state ! --state NEW -j STATENOTNEW");
+				if ($ipv6) {
+					push (@$Listing, "-A STATE$_ ! -p ipv6-icmp -m state ! --state NEW -j STATENOTNEW");
+				} else {
+					push (@$Listing, "-A STATE$_ -m state ! --state NEW -j STATENOTNEW");
+				}
 				push (@$Listing, "-A STATELESS$_ -j ACCOUNTINGSTATELESS$_");
 			}
 			push (@$Listing, "-A STATENOTNEW -m limit --limit $$Sysconfig{'LogLimit'} --limit-burst $$Sysconfig{'LogBurst'} -j LOG --log-prefix \"$$Sysconfig{'LogPrefix'} STATE NOT NEW: \"  --log-level $$Sysconfig{'LogLevel'} --log-tcp-options --log-ip-options");
 			push (@$Listing, "-A STATENOTNEW -j DROP");
 			push (@$Listing, "-A MYREJECT -m tcp -p tcp -j REJECT --reject-with tcp-reset");
-			push (@$Listing, "-A MYREJECT -j REJECT --reject-with icmp-port-unreachable");
+			if ($ipv6) {
+				push (@$Listing, "-A MYREJECT -j REJECT --reject-with icmp6-port-unreachable");
+			} else {
+				push (@$Listing, "-A MYREJECT -j REJECT --reject-with icmp-port-unreachable");
+			}
 		} elsif ($entry eq 'nat') {
 			$table=\@nat;
 			$chains=\%nat;
@@ -1302,8 +1363,11 @@ sub applyRules {
 	my $error;
 
 	@$Listing=map { $_."\n" } @$Listing;
-
-	open (IPT, '/sbin/iptables-save|');
+	if ($ipv6) {
+		open (IPT, '/sbin/ip6tables-save|');
+	} else {
+		open (IPT, '/sbin/iptables-save|');
+	}
 	@oldrules = <IPT>;
 	close (IPT);
 
@@ -1312,7 +1376,11 @@ sub applyRules {
 	$SIG{'QUIT'} = 'signalCatcher';
 	$SIG{'TERM'} = 'signalCatcher';
 
-	open (IPT, '|/sbin/iptables-restore');
+	if ($ipv6) {
+		open (IPT, '|/sbin/ip6tables-restore');
+	} else {
+		open (IPT, '|/sbin/iptables-restore');
+	}
 	print IPT @$Listing;
 	close (IPT);
 	$error=$?;
@@ -1321,7 +1389,11 @@ sub applyRules {
 		sleep $timeout;
 	}
 	if ($timeout || $SignalCatched || $error) {
-		open (IPT, '|/sbin/iptables-restore');
+		if ($ipv6) {
+			open (IPT, '|/sbin/ip6tables-restore');
+		} else {
+			open (IPT, '|/sbin/iptables-restore');
+		}
 		print IPT @oldrules;
 		close (IPT);
 		if ($SignalCatched) {
@@ -1369,7 +1441,7 @@ sub readCommandLine {
 	}
 	if (exists($ENV{'LOGLEVEL'})) {
 		$Sysconfig{'LogLevel'}=$ENV{'LOGLEVEL'};
-	} else { 
+	} else {
 		$Sysconfig{'LogLevel'}='debug';
 	}
 	if (exists($ENV{'LOGPREFIX'})) {
@@ -1393,8 +1465,10 @@ sub readCommandLine {
 		$Sysconfig{'AccountPrefix'}='ACC_';
 	}
 	my %opt;
-	getopts('c:tpds:b:r:T:C:R:D:w:W', \%opt) || uifUsg ();
-	
+	getopts('6c:tpds:b:r:T:C:R:D:w:W', \%opt) || uifUsg ();
+
+	$ipv6 = 1 if $opt{'6'};
+	$configfile=$configfile6 if $opt{'6'};
 	$configfile = $opt{'c'} if $opt{'c'};
 	$test = 1 if $opt{'t'};
 	$print = 1 if $opt{'p'};
@@ -1437,6 +1511,14 @@ sub readCommandLine {
 		chomp($ldappassword);
 	}
 
+	if ($ipv6) {
+		if (exists($ENV{'LOGPREFIX6'})) {
+			$Sysconfig{'LogPrefix'}=$ENV{'LOGPREFIX6'};
+		} else {
+			$Sysconfig{'LogPrefix'}='FW6';
+		}
+	}
+
 	if ($readldap || $writeldap) {
 		if ($LDAPENABLED == 0) { die "To use LDAP fatures be sure to install Net::LDAP from debain package libnet-ldap-perl" } ;
 		$ldap = Net::LDAP->new($ldapserver) or die "$@";
@@ -1490,11 +1572,13 @@ sub clearAllRules {
 	push (@$Listing, ":PREROUTING ACCEPT [0:0]");
 	push (@$Listing, ":OUTPUT ACCEPT [0:0]");
 	push (@$Listing, "COMMIT");
-	push (@$Listing, "*nat");
-	push (@$Listing, ":PREROUTING ACCEPT [0:0]");
-	push (@$Listing, ":POSTROUTING ACCEPT [0:0]");
-	push (@$Listing, ":OUTPUT ACCEPT [0:0]");
-	push (@$Listing, "COMMIT");
+	if ($ipv6) {} else {
+		push (@$Listing, "*nat");
+		push (@$Listing, ":PREROUTING ACCEPT [0:0]");
+		push (@$Listing, ":POSTROUTING ACCEPT [0:0]");
+		push (@$Listing, ":OUTPUT ACCEPT [0:0]");
+		push (@$Listing, "COMMIT");
+	}
 	push (@$Listing, "*filter");
 	push (@$Listing, ":INPUT ACCEPT [0:0]");
 	push (@$Listing, ":OUTPUT ACCEPT [0:0]");
@@ -1503,8 +1587,9 @@ sub clearAllRules {
 }
 
 sub uifUsg {
-	print "usage: $0 [-c configfile] [-t] [-p] [-d] [-s server] [-b base] [-r ruleset] [-R ruleset] [-D <bind dn>] [-W] [-w <password>] [-T time] [-C configfile]\n";
-	print "-c  read <configfile> instead of $configfile\n";
+	print "usage: $0 [-6] [-c configfile] [-t] [-p] [-d] [-s server] [-b base] [-r ruleset] [-R ruleset] [-D <bind dn>] [-W] [-w <password>] [-T time] [-C configfile]\n";
+	print "-6  ipv6 mode default config $configfile6\n";
+	print "-c  read <configfile> instead of $configfile (or in ipv6 mode $configfile6)\n";
 	print "-t  test rules\n";
 	print "-p  print rules to stdout\n";
 	print "-d  disable firewall (clear all rules)\n";
@@ -1684,7 +1769,7 @@ sub readLdap {
 		my $arrayref = $$result{$entry};
 		my $name = @{$$arrayref{'cn'}}[0];
 		if (exists($fwruleset{$name})) {
-			my $type=@{$$arrayref{'uiftype'}}[0]; 
+			my $type=@{$$arrayref{'uiftype'}}[0];
 			my %temphash;
 			$temphash{'Id'} = $name;
 			if ($type =~ /^\s*(\w+([-+|>]|{\w+}))$/) {
@@ -1701,7 +1786,7 @@ sub readLdap {
 					}
 				}
 			}
-			$$Rules[$fwruleset{$name}]=\%temphash;	
+			$$Rules[$fwruleset{$name}]=\%temphash;
 		}
 	}
 }
@@ -1768,7 +1853,7 @@ sub writeLdap {
 			push (@ruleset, $$Sysconfig{$syskeys});
 		}
 		addLdap ($ldap, "cn=$writeldapruleset,ou=RuleSets,ou=Filter,ou=Sysconfig,$ldapbase", \@ruleset);
-	}	
+	}
 	my $network;
 	foreach $network (keys (%$Networks)) {
 		my $counter="";
